%%%%% solving the stochastic growth  model with elastic labor supply using collocation

clearvars;
close all;
clc;

%%%%% economic parameters

alpha  = 0.35;       %% capital's share in production function
beta   = 1/1.01;     %% time discount factor   
delta  = 0.03;       %% depreciation rate
sigma  = 1;          %% CRRA (=1/IES)
varphi = 1;          %% inverse Frisch elasticity labor supply

zbar  = 1;
rhoz  = 0.98;        %% AR1 coefficient, productivity shocks
sigz  = 0.015;       %% innovation std dev, productivity shocks

%%%%% numerical parameters

max_iter = 500;      %% maximum number of iterations
tol      = 1e-7;     %% treat numbers smaller than this as zero

do_irf   = 1;        %% turn off/on irf and sim
do_sim   = 1;

%%%%% grid for productivity shocks

nz       = 29; %7;        %% number of breakpoints for z grid
nez      = 15; %5;        %% number of nodes for quadrature (shocks)

% quadrature nodes and weights
[ez, wz] = qnwunif(nez,1e-9,1-1e-9);
ez       = sigz*norminv(ez,0,1); 

zmin     = exp(-4*sqrt(1/(1-rhoz^2))*sigz);
zmax     = exp( 4*sqrt(1/(1-rhoz^2))*sigz);
zgrid    = exp(nodeunif(nz, log(zmin), log(zmax)));

%%%%% grid for capital stock

nk       = 99;       %% number of breakpoints for k grid
curv     = 0.25;     %% (curv = 0 log-spaced, curv = 1 linear)

kmin     = 1e-3;
kmax     = (zmax/delta)^(1/(1-alpha));
kgrid    = nodeunif(nk, kmin.^curv, kmax.^curv).^(1/curv);

N        = 1001;     %% number of nodes for fine grid (see below)

%%%%% put in a structure to pass to other functions

parameters.alpha  = alpha;
parameters.beta   = beta;
parameters.delta  = delta;
parameters.sigma  = sigma;
parameters.varphi = varphi;
parameters.rhoz   = rhoz;
parameters.sigz   = sigz;

parameters.ez     = ez;
parameters.wz     = wz;

%%%%% setup state space using CompEcon tools

fspace   = fundef({'spli', kgrid},...
                  {'spli', zgrid}); % function space structure

grid     = funnode(fspace);% nodes where we solve the problem
Phi      = funbas(fspace); % matrix of collocation basis vectors 
                           % Phi_{ij} = phi_j(k_i) 

kgrid    = grid{1}; % extra 2 points for 3rd-order spline
zgrid    = grid{2}; % extra 2 points for 3rd-order spline

kmin     = kgrid(1);
kmax     = kgrid(end);

zmin     = zgrid(1); 
zmax     = zgrid(end); 

%%%%% bounds on consumption

cmin     = tol;
cmax     = zmax*(kmax.^alpha) + (1-delta)*kmax - eps;

%%%%% form collection of states

s        = gridmake(grid); % ns-by-2 matrix where ns=nk*nz
ns       = size(s,1);        

k        = s(:,1);
z        = s(:,2);

%%%%% initial guess at collocation coefficients "a"

c = alpha*beta*z.*k.^alpha; % guess for consumption policy
v = log(c)/(1-beta);        % guess for value function

a = Phi\v;                  % implied collocation coefficients

tic
%%%%% solve Bellman equation

for i=1:max_iter;

%%%%% optimal consumption given these coefficints

c = solve_brent('rhs_bellman',s,parameters,a,fspace,cmin,cmax,tol);

%%%%% maximized rhs of Bellman equation

v = rhs_bellman(c,s,parameters,a,fspace); %% v(a)

%%%%% update collocation coefficients
Jacobian = 0;

%%%%% implied by optimal consumption

l = ((1-alpha).*z.*(k.^alpha)./c).^(1/(varphi+alpha));

kprime = z.*(k.^alpha).*(l.^(1-alpha)) + (1-delta)*k-c;

%%%%% build up Jacobian matrix of v(a)
for j=1:numel(ez),

zprime = max(min(z.^rhoz.*exp(ez(j)), zmax), zmin);

sprime = [kprime,zprime];

Jacobian = Jacobian+beta*wz(j)*funbas(fspace,sprime);

end

%%%%% Newton's method
anew = a - (Phi-Jacobian)\(Phi*a-v); 

%%%%% check if converged

error = norm(anew-a,inf);

fprintf('%4i %6.2e \n',[i, error]);

if error<tol, break, end;

%%%%% if not converged, update and try again

a = anew;

end


time      = toc;
last_iter = i;

if error<tol && last_iter<=max_iter,
   
fprintf('\n');    
fprintf('time taken (seconds) = %3.3f \n',time);
fprintf('number of iterations = %3.0f \n',last_iter);
fprintf('error at last step   = %3.10f \n',error);
fprintf('\n'); 

end

if do_irf==1,

%%%%% interpolation coefficients

c_coeff = funfitxy(fspace,s,c);          %% for c(k,z)
g_coeff = funfitxy(fspace,s,kprime);     %% for g(k,z)=k'

%%%%% non-stochastic steady state values

rho  = 1/beta-1;

lbar = ((1-alpha)*(rho+delta)/(rho+(1-alpha)*delta))^(1/(1+varphi));
kbar = ((alpha*zbar/(rho+delta))^(1/(1-alpha)))*lbar;
ybar = zbar*(kbar^alpha)*(lbar^(1-alpha));
ibar = delta*kbar;
cbar = ybar - ibar;

%%%%% limit of policy function

kt = zeros(1000,1);

for t=1:numel(kt),

    kt(1) = kbar;
    
    %%%%% iterate on policy function
    
    kt(t+1) = funeval(g_coeff,fspace,[kt(t),zbar]);
    
end

kbar = kt(end);

%%%%% impulse response

T    = 250; 
time = 0:1:T;

z0      = sigz;
zt      = (z0*rhoz.^time)'; 
zt      = zbar*exp(zt);       

kt      = zeros(size(zt));

for t=1:T,

    kt(1) = kbar;
    
    %%%%% iterate on policy function
    
    kt(t+1) = funeval(g_coeff,fspace,[kt(t),zt(t)]);
    
end

ct = funeval(c_coeff,fspace,[kt,zt]);
lt = ((1-alpha).*zt.*(kt.^alpha)./ct).^(1/(varphi+alpha));
yt = zt.*(kt.^alpha).*(lt.^(1-alpha));
it = yt-ct;

%%%% put in log deviations form

zt = log(zt/zt(end));
kt = log(kt/kt(end));
ct = log(ct/ct(end));
yt = log(yt/yt(end)); 
it = log(it/it(end)); 
lt = log(lt/lt(end)); 
 

figure(1)
plot(time,zt,'b-' ,time,kt,'--',time,ct,'g-',time,yt,'r-',time,lt,'m-',time,zeros(size(time)),'k--');
legend('productivity','capital','consumption','output','labor','Location','Best')
xlabel('periods after shock')
ylabel('log-deviation') 

end

if do_sim==1,

%%%%% simulation

rng('default'); %% set seed of random number generator

T    = 1000; 
time = 0:1:T;

shock    = randn(size(time))*sigz;

for t=1:T,
    
    zt(1) = 0;
    zt(t+1) = rhoz*zt(t)+shock(t+1);
    
end

zt      = zbar*exp(zt);
kt      = zeros(size(zt));

for t=1:T,

    kt(1) = kbar;
    
    %%%%% iterate on policy function
    
    kt(t+1) = funeval(g_coeff,fspace,[kt(t),zt(t)]);
    
end

ct = funeval(c_coeff,fspace,[kt,zt]);
lt = ((1-alpha).*zt.*(kt.^alpha)./ct).^(1/(varphi+alpha));
yt = zt.*(kt.^alpha).*(lt.^(1-alpha));
it = yt-ct;

%%%% put in log deviations form

zt = log(zt/mean(zt));
kt = log(kt/mean(kt));
ct = log(ct/mean(ct));
yt = log(yt/mean(yt)); 
it = log(it/mean(it)); 
lt = log(lt/mean(lt)); 


%%%%% moments

X = [ct,kt,lt,yt,zt];

std(X)

corrcoef(X)

figure(2)
plot(time,zt,'b-' ,time,kt,'--',time,ct,'g-',time,yt,'r-',time,lt,'m-',time,zeros(size(time)),'k--');
legend('productivity','capital','consumption','output','employment','Location','Best')
xlabel('time')
ylabel('log-deviation from long-run mean') 

end


%return

%%%%% reshape solution

Nk = numel(kgrid);
Nz = numel(zgrid);

VV = reshape(v,Nk,Nz); %% VV = v(k_i,z_j)
CC = reshape(c,Nk,Nz); %% CC = c(k_i,z_j)
LL = reshape(l,Nk,Nz); %% LL = l(k_i,z_j)

%%%%% optimal policy k'=g(k,z)
GG = reshape(kprime,Nk,Nz); %% GG = g(k_i,z_j)

figure(3)
subplot(2,2,1)
plot(kgrid,VV,'b-')
xlabel('capital stock, k')
ylabel('value function, v(k,z)')
axis([0 350 -20 100])

subplot(2,2,2)
plot(kgrid,GG,'r-',k,k,'k--')
xlabel('capital stock, k')
ylabel('capital accumulation, g(k,z)')
axis([0 350 0 350])

subplot(2,2,3)
plot(kgrid,CC,'g-')
xlabel('capital stock, k')
ylabel('consumption, c(k,z)')
axis([0 350 0 10])

subplot(2,2,4)
plot(kgrid,LL,'m-')
xlabel('capital stock, k')
ylabel('labor, l(k,z)')
axis([0 350 0 1.4])






